use std::sync::Arc;

use crate::enums::ProtocolVersion;

use super::command_base::{CommandDeserializer, DeserializedCommand};

pub const DESERIALIZE_VERSION_RAW_NAME: &str = "_ver";

#[derive(Debug)]
pub struct VersionCommand {
    pub version: ProtocolVersion,
}

impl DeserializedCommand for VersionCommand {
    fn raw_name(&self) -> &'static str {
        DESERIALIZE_VERSION_RAW_NAME
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) {
        state.info.api_version = self.version;
    }
}

#[derive(Default)]
pub struct VersionCommandDeserializer {}

impl CommandDeserializer for VersionCommandDeserializer {
    fn deserialize(&self, buffer: &[u8]) -> std::sync::Arc<dyn DeserializedCommand> {
        let version = u32::from_be_bytes([buffer[0], buffer[1], buffer[2], buffer[3]]);
        let version: ProtocolVersion = version.try_into().expect("Invalid protocol version");

        Arc::new(VersionCommand { version })
    }
}
